@@ -1,6 +1,5 @@
 context("Convenience functions")
 
-
 test_that("convenience functions work as expected", {
   
   # argmax/argmin
@@ -10,25 +9,35 @@ test_that("convenience functions work as expected", {
   expect_identical(argmin(a), c(1L, 1L))
   expect_identical(argmin(a, rows = FALSE), c(1L, 1L, 1L))
   
+  # atleast_2d
+  expect_identical(dim(atleast_2d(1:5)), c(5L, 1L))
+  expect_identical(dim(atleast_2d(a)), c(2L, 3L))
+
+  # clip
+  x <- 1:9
+  expect_identical(clip(x, 3, 7), c(3, 3, 3, 4, 5, 6, 7, 7, 7))
+  expect_identical(clip(matrix(x, nrow = 3, ncol = 3), 3, 7), cbind(3, 4:6, 7))
+
   # eye
   d <- eye(3, 3)
   expect_identical(d, diag(1L, 3, 3))
-  expect_identical(d, inv(d))  # inverse of identity is identity
+  expect_equal(d, inv(d))  # inverse of identity is identity  # FIXME: Why not identiacal anymore?
   
-  # fill (ones, zeros, trues, falses), rand, randi, randn
+  # fill rand, randi, randn
   expect_equal(dim(fill(0L, 10)), c(10, 1))
-  expect_equal(dim(randn(10)), c(10, 1))
-  expect_equal(dim(rand(10)), c(10, 1))
-  expect_equal(dim(randi(imax = 100, 10)), c(10, 1))
   expect_null(dim(fill(0L, 10, atleast_2d = FALSE)))
-  expect_null(dim(randn(10, atleast_2d = FALSE)))
-  expect_null(dim(rand(10, atleast_2d = FALSE)))
-  expect_null(dim(randi(imax = 100, 10, atleast_2d = FALSE)))
-  expect_that(dim(fill(0L, 2, 2, 2)), equals(c(2, 2, 2)))
-  expect_that(dim(rand(2, 2, 2)), equals(c(2, 2, 2)))
-  expect_that(dim(randi(imax = 100, 2, 2, 2)), equals(c(2, 2, 2)))
-  expect_that(dim(randn(2, 2, 2)), equals(c(2, 2, 2)))
+  expect_equal(dim(fill(0L, 2, 2, 2)), c(2, 2, 2))
+
   
+  # falses, trues, ones, and zeros
+  expect_identical(falses(10), atleast_2d(rep(FALSE, 10)))
+  expect_identical(trues(10), atleast_2d(rep(TRUE, 10)))
+  expect_identical(trues(10), !atleast_2d(rep(FALSE, 10)))
+  expect_identical(trues(10), atleast_2d(!rep(FALSE, 10)))
+  expect_identical(trues(10), atleast_2d(rep(!FALSE, 10)))
+  expect_identical(ones(10), atleast_2d(rep(1L, 10)))
+  expect_identical(zeros(10), atleast_2d(rep(0L, 10)))
+
   # flatten
   m1 <- matrix(1:9, 3, 3, byrow = TRUE)
   m2 <- matrix(1:9, 3, 3, byrow = FALSE)
@@ -38,8 +47,11 @@ test_that("convenience functions work as expected", {
   
   # inv, tr
   expect_error(inv(fill(3L, 2, 2)))  # singular matrix
+  expect_error(inv(randn(3, 2)))  # non-square matrix
+  expect_error(inv(as.data.frame(randn(s, 2))))  # data frame
   expect_identical(inv(2 * eye(3, 3)), 0.5 * eye(3, 3))
   expect_identical(tr(5 * eye(3, 4)), 15)
+  expect_identical(tr(diag(8)), 8)
   
   # hcat, vcat
   m1 <- randn(2, 3)
@@ -48,6 +60,10 @@ test_that("convenience functions work as expected", {
   expect_identical(vcat(m1, m2), rbind(m1, m2))
   
   # linspace, logspace
+  x <- linspace(1, 10, 20)
+  y <- logspace(1, 10, 20)
+  expect_identical(x, seq(from = 1, to = 10, length.out = 20))
+  expect_identical(y, 10^x)
   
   # meshgrid
   x <- linspace(0, 1, 3)
@@ -60,15 +76,45 @@ test_that("convenience functions work as expected", {
   z2 <- outer(x, y, function(x, y) sin(x^2 + y^2) / (x^2 + y^2))
   expect_identical(z1, z2)
   
+  # rand, randi, and randn
+  expect_equal(dim(rand(2, 2, 2)), c(2, 2, 2))
+  expect_equal(dim(randi(imax = 100, 2, 2, 2)),c(2, 2, 2))
+  expect_equal(dim(randn(2, 2, 2)), c(2, 2, 2))
+  expect_null(dim(randn(10, atleast_2d = FALSE)))
+  expect_null(dim(rand(10, atleast_2d = FALSE)))
+  expect_null(dim(randi(imax = 100, 10, atleast_2d = FALSE)))
+  expect_equal(dim(randn(10)), c(10, 1))
+  expect_equal(dim(rand(10)), c(10, 1))
+  expect_equal(dim(randi(imax = 100, 10)), c(10, 1))
+  expect_error(randi(imax = -100, 10))
+  
+  # repmat
+  m1 <- mat("1, 2; 3, 4")
+  m2 <- mat("1, 2, 1, 2; 3, 4, 3, 4; 1, 2, 1, 2; 3, 4, 3, 4")
+  expect_identical(repmat(m1, 2, 2), m2)
+  
   # resize, size
   
   # tri, tril, triu, is.tril, is.triu
-
+  m <- mat("1, 2, 3; 4, 5, 6; 7, 8, 9")
+  mu <- mat("1, 2, 3; 0, 5, 6; 0, 0, 9")
+  mu2 <- mat("0, 2, 3; 0, 0, 6; 0, 0, 0")
+  ml <- mat("1, 0, 0; 4, 5, 0; 7, 8, 9")
+  ml2 <- mat("0, 0, 0; 4, 0, 0; 7, 8, 0")
+  expect_identical(triu(m, diag = TRUE), mu)
+  expect_identical(triu(m, diag = FALSE), mu2)
+  expect_identical(tril(m, diag = TRUE), ml)
+  expect_identical(tril(m, diag = FALSE), ml2)
+  expect_true(is.triu(mu))
+  expect_true(is.triu(mu2))
+  expect_true(is.tril(ml))
+  expect_true(is.tril(ml2))
+  
   # Resize a vector into an array
   x <- 1:8
   a <- resize(1:8, 2, 2, 2)
-  expect_that(a, is_a("array"))
-  expect_that(flatten(a), is_identical_to(x))
+  expect_is(a, "array")
+  expect_identical(flatten(a), x)
   
 
   # Meshgrid (identical)
@@ -76,7 +122,7 @@ test_that("convenience functions work as expected", {
   mg <- meshgrid(x, y)
   z1 <- sin(mg[[1]]^2 + mg[[2]]^2) / (mg[[1]]^2 + mg[[2]]^2)
   z2 <- outer(x, y, function(x, y) sin(x^2 + y^2) / (x^2 + y^2))
-  expect_that(z1, is_identical_to(z2))
+  expect_identical(z1, z2)
   
   # Triangular matrices
   m1 <- mat("1, 1, 1, 0, 0; 
@@ -85,9 +131,9 @@ test_that("convenience functions work as expected", {
   m2 <- mat("0, 0, 0, 0, 0; 
              1, 0, 0, 0, 0; 
              1, 1, 0, 0, 0")
-  expect_that(tri(3, 5, k = 2), equals(m1, check.attributes = FALSE))
-  expect_that(tri(3, 5, k = -1), equals(m2, check.attributes = FALSE))                       
-  expect_that(tri(3, 5, diag = FALSE), equals(m2, check.attributes = FALSE))
+  expect_equal(tri(3, 5, k = 2), m1, check.attributes = FALSE)
+  expect_equal(tri(3, 5, k = -1), m2, check.attributes = FALSE)                     
+  expect_equal(tri(3, 5, diag = FALSE), m2, check.attributes = FALSE)
   
   m3 <- matrix(1:12, nrow = 4, ncol = 3, byrow = TRUE)
   m4 <- mat(" 0,  0,  0; 
